#include "my_spawn.h"
#include "boost/process.hpp"
#include <fstream>
#include <iostream>


class status_getter {
    std::string filename;
public:
    status_getter(const std::string& pid);
    std::string operator()() const;
};


std::string grab_pid(const std::string& prog_name);


namespace bp = boost::process;

int main(int argc, char**) {
    const char* program_name = "/bin/sleep";
    const int timeout = 5;
    const auto tm_str = std::to_string(timeout);

    if (argc > 1) {
        std::cout << "## RUNNING DOUBLE FORK DEMO ##" << std::endl;
        char* args[] = {const_cast<char*>(program_name),
                        const_cast<char*>(tm_str.c_str()),
                        nullptr};
        my_spawn(args);
    }
    else {
        std::cout << "## RUNNING BOOST SPAWN ##" << std::endl;
        bp::spawn(program_name, bp::args({tm_str}));

        /// obvious fix but it makes the program ignore all children
        /// a possible mitigation is to use double ::fork()
        //signal(SIGCHLD, SIG_IGN);
    }

    status_getter getter(grab_pid(program_name));

    for (int i = 0; i < timeout*2; ++i) {
        std::cout << getter() << std::endl;
        sleep(1);
    }
}


status_getter::status_getter(const std::string &pid) :
    filename("/proc/" + pid + "/stat")
{}


std::string status_getter::operator()() const {
    std::ifstream file(filename);
    if (file.is_open()) {
        std::string word;
        if (file >> word >> word >> word) {
            return "state: " + word;
        }
        return "empty stat file";
    }
    return "process does not exist";
}


std::string grab_pid(const std::string& prog_name) {
    const std::string command {"pidof " + prog_name};
    FILE* f = popen(command.c_str(), "r");

    char buf[1024];
    int cnt = 0;
    while (fscanf(f, " %1023s", buf) == 1) cnt++;
    pclose(f);

    if (cnt < 1) {
        std::cout << "no program called '" +
                     prog_name + "'" << std::endl;
        exit(EXIT_FAILURE);
    }
    else if (cnt > 1) {
        std::cout << "more than one program has the name '" +
                     prog_name + "'" << std::endl;
        exit(EXIT_FAILURE);
    }

    return buf;
}
